/* This file is a part of the SimpleScalar tool suite written by
 * Todd M. Austin, University of Wisconsin - Madison, Computer Sciences
 * Department as a part of the Multiscalar Research Project.
 *  
 * The SimpleScalar x86 port was written by Steve Bennett.
 * 
 * The tool suite is currently maintained by Doug Burger.
 * 
 * Copyright (C) 1994, 1995, 1996 by Todd M. Austin
 *
 * This source file is distributed "as is" in the hope that it will be
 * useful.  The tool set comes with no warranty, and no author or
 * distributor accepts any responsibility for the consequences of its
 * use. 
 * 
 * Everyone is granted permission to copy, modify and redistribute
 * this tool set under the following conditions:
 * 
 *    This source code is distributed for non-commercial use only. 
 *    Please contact the maintainer for restrictions applying to 
 *    commercial use.
 *
 *    Permission is granted to anyone to make or distribute copies
 *    of this source code, either as received or modified, in any
 *    medium, provided that all copyright notices, permission and
 *    nonwarranty notices are preserved, and that the distributor
 *    grants the recipient permission for further redistribution as
 *    permitted by this document.
 *
 *    Permission is granted to distribute this file in compiled
 *    or executable form under the same conditions that apply for
 *    source code, provided that either:
 *
 *    A. it is accompanied by the corresponding machine-readable
 *       source code,
 *    B. it is accompanied by a written offer, with no time limit,
 *       to give anyone a machine-readable copy of the corresponding
 *       source code in return for reimbursement of the cost of
 *       distribution.  This written offer must permit verbatim
 *       duplication by anyone, or
 *    C. it is distributed by someone who received only the
 *       executable form, and is accompanied by a copy of the
 *       written offer of source code that they received concurrently.
 *
 * In other words, you are welcome to use, share and improve this
 * source file.  You are forbidden to forbid anyone else to use, share
 * and improve what you give them.
 *
 * INTERNET: dburger@cs.wisc.edu
 * US Mail:  1210 W. Dayton Street, Madison, WI 53706
 */

/***********************************************************************
    INCLUDES
 ***********************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <varargs.h>
#if(0)
#include <stdarg.h>
#endif

#include <string.h>

#include "misc.h"
#include "errorRoutines.h"

/* debug flag */
int            verbose = FALSE;
int            syscallVerbose = FALSE;
long long      numInsts = 0; 		/* speculative instructions executed */
long long      nnumInsts = 0; 		/* non-speculative instructions executed */
long long      numCycles = 0; 		/* issue stats */
long long      numDecodeCacheHits = 0;
time_t         startTime; 		/* start time in seconds */
jmp_buf        exitBuf; 		/* jump to this at exit time */
enum SimType   simType = Functional; 	/* simulation type requested */
int            dumpStats = FALSE; 	/* set this for intermediate stats */
unsigned long  verbInst = 0; 		/* when to get verbose (instructions) */
unsigned long  verbAddr = 0; 		/* when to get verbose (instruction address) */
unsigned long  verbTime = 0; 		/* when to get verbose (cycle number) */
unsigned long  lastSyscall = 0; 	/* for debugging condor problems */

/***********************************************************************
 ***********************************************************************/
void
_info(char *file, char *func, int line, char *fmt, ...)
{
	va_list v;
	va_start(v);

	fprintf(stderr, "** ");
	vfprintf(stderr, fmt, v);
	fprintf(stderr, "\n");

}/*** end _info() ***/




#ifndef NDEBUG
/***********************************************************************
 ***********************************************************************/
void
_debug(char *file, char *func, int line, char *fmt, ...)
{
	va_list v;
	va_start(v);

	fprintf(stderr, "** Debug: %s{%s,%d}: ", func, file, line);
	vfprintf(stderr, fmt, v);
	fprintf(stderr, "\n");

}/*** end _debug() ***/
#endif /* NDEBUG */


/***********************************************************************
 * copy a string to a new storage allocation
 ***********************************************************************/
char *
strdup(const char *s)
{
	char *newBuf;

	if (!(newBuf = (char *)malloc(strlen(s)+1)))
		return NULL;
	strcpy(newBuf, s);
	return newBuf;
}/*** end strdup() ***/



/***********************************************************************
 * allocate some core, this memory has overhead no larger than a page
 * in size and it cannot be released. the storage is returned cleared
 ***********************************************************************/
void *
GetCore(unsigned long size)
{
#ifndef PURIFY
	void *p = sbrk((signed)size);

	if (p == (void *)-1)
		return NULL;

	/* this may be superfluous */
	bzero(p, (signed)size);
	return p;
#else
	return calloc(size, 1);
#endif /* PURIFY */

}/*** end GetCore() ***/




/***********************************************************************
 * return string describing elapsed time, passed in SEC in seconds
 ***********************************************************************/
char *
ElapsedTime(long sec)
{
	static char tStr[256];
	char temp[256];

	if (sec <= 0)
		return "0s";

	tStr[0] = '\0';

	/* days */
	if (sec >= 86400) {
		sprintf(temp, "%ldD ", sec/86400);
		strcat(tStr, temp);
		sec = sec % 86400;
	}
	/* hours */
	if (sec >= 3600) {
		sprintf(temp, "%ldh ", sec/3600);
		strcat(tStr, temp);
		sec = sec % 3600;
	}
	/* mins */
	if (sec >= 60) {
		sprintf(temp, "%ldm ", sec/60);
		strcat(tStr, temp);
		sec = sec % 60;
	}
	/* secs */
	if (sec >= 1) {
		sprintf(temp, "%lds ", sec);
		strcat(tStr, temp);
	}
	tStr[strlen(tStr)-1] = '\0';
	return tStr;

}/*** end ElapsedTime() ***/





/*
 * get option letter from argument vector
 */
#define	BADCH	(int)'?'
#define	EMSG	""

char	    *optArg;		/* argument associated with option */
int	    optErr = TRUE;	/* if error message should be printed */
int	    optInd = 1;		/* index into parent argv vector */
int	    optOpt;		/* character checked for validity */
static char *place = EMSG;	/* option letter processing */

/***********************************************************************
 ***********************************************************************/
void
NewOpt(void)
{
	optArg = NULL;
	optErr = TRUE;
	optInd = 1;
	optOpt = 0;
	place = EMSG;

}/*** end NewOpt() ***/

/***********************************************************************
 ***********************************************************************/
char
GetOpt(int nargc, char **nargv, char *ostr)
{
	register char *oli;			/* option letter list index */
	char *p;

	if (!*place) {				/* update scanning pointer */
		if (optInd >= nargc || *(place = nargv[optInd]) != '-') {
			place = EMSG;
			return (EOF);
		}
		if (place[1] && *++place == '-') {	/* found "--" */
			++optInd;
			place = EMSG;
			return (EOF);
		}
	}					/* option letter okay? */
	if ((optOpt = (int)*place++) == (int)':' ||
	    !(oli = strchr(ostr, optOpt))) {
    
		/*
		 * For backwards compatibility: don't treat '-' as an
		 * option letter unless caller explicitly asked for it.
		 */
		if (optOpt == (int)'-')
			return (EOF);
		if (!*place)
			++optInd;
		if (optErr) {
			if (!(p = strrchr(*nargv, '/')))
				p = *nargv;
			else
				++p;
			(void)fprintf(stderr, "%s: illegal option -- %c\n", p, optOpt);
		}
		return (BADCH);
	}
	if (*++oli != ':') {			/* don't need argument */
		optArg = NULL;
		if (!*place)
			++optInd;
	} else {				/* need an argument */
		if (*place)			/* no white space */
			optArg = place;
		else if (nargc <= ++optInd) {	/* no arg */
			place = EMSG;
			if (!(p = strrchr(*nargv, '/')))
				p = *nargv;
			else
				++p;
			if (optErr)
				(void)fprintf(stderr, "%s: option requires an argument -- %c\n",
					      p, optOpt);
			return (BADCH);
		} else				/* white space */
			optArg = nargv[optInd];
		place = EMSG;
		++optInd;
	}
	return (optOpt);			/* dump back option letter */

}/*** end getOpt() ***/




/***********************************************************************
 ***********************************************************************/
Bit16u
zeroExtend8_16( Bit8u byte)
{
	Bit16u  retInt = 0x0000;

	retInt = (Bit16u)(byte) & 0x00ff;

	return retInt;

}/*** end zeroExtend8_16() ***/



/***********************************************************************
 ***********************************************************************/
Bit32u
zeroExtend8_32( Bit8u byte)
{
	Bit32u  retInt = 0x00000000;

	retInt = (Bit32u)(byte) & 0x000000ff;

	return retInt;

}/*** end zeroExtend8_32() ***/




/***********************************************************************
 ***********************************************************************/
Bit32u
zeroExtend16_32( Bit16u word)
{
	Bit32u  retInt = 0x00000000;

	retInt = (Bit32u)(word) & 0x0000ffff;

	return retInt;

}/*** end zeroExtend16_32() ***/




/***********************************************************************
 ***********************************************************************/
Bit16u
signExtend8_16( Bit8u byte)
{
	int     highBit;
	Bit16u  retInt = 0x0000;

	highBit =  (byte & (1<<7));
	
	if (highBit == 0)
		retInt = (Bit16u)(byte) & 0x00ff;
	else
		retInt = (Bit16u)(byte) | 0xff00;

	return retInt;

}/*** end signExtend8_16() ***/




/***********************************************************************
 ***********************************************************************/
Bit32u
signExtend8_32( Bit8u byte)
{
	int     highBit;
	Bit32u  retInt = 0x00000000;

	highBit =  (byte & (1<<7));

	if (highBit == 0)
		retInt = (Bit32u)(byte) & 0x000000ff;
	else
		retInt = (Bit32u)(byte) | 0xffffff00;

	return retInt;

}/*** end signExtend8_32() ***/




/***********************************************************************
 ***********************************************************************/
Bit32u
signExtend16_32( Bit16u word)
{
	int     highBit;
	Bit32u  retInt = 0;

	highBit =  (word & (1<<15));

	if (highBit == 0)
		retInt = (Bit32u)(word) & 0x0000ffff;
	else
		retInt = (Bit32u)(word) | 0xffff0000;
	
	return retInt;

}/*** end signExtend16_32() ***/
